--
-- visible
--
-- @author Tom Wießner
-- @date 03/03/16 
--
-- @lastEdit 20/12/16
-- @changes add helptext
--
-- specialisation to change Visibility Events
-- spezialisation zum wechseln eines Sichtbarkeitsevent
--
-- (c) by Tom Wießner, all rights reserved!
-- using only with premission from Tom Wießner!
-- support at http://mountainmodding.tk
--
-- this script isn´t releasereddy!
 
visible = {};

function visible.prerequisitesPresent(specializations)
    return true;
end;

function visible:load(xmlFile)
	self.swapVisible = SpecializationUtil.callSpecializationsFunction("swapVisible");
	
	self.visibleNodes = {};
	local i=0;
	while true do
	    -- load basic Strings
		local str1 = getXMLString(xmlFile, string.format("vehicle.visible.visible(%d)#objectHIndex", i));
		local input = getXMLString(xmlFile, string.format("vehicle.visible.visible(%d)#inputName",i));
		local help = Utils.getNoNil(getXMLBool(xmlFile,"vehicle.visible.visible(%d)#helpText", false));
		local ONS = getXMLBool(xmlFile, string.format("vehicle.visible.visible(%d)#objectNState",i));
		local errorInformations = getXMLString(xmlFile, string.format("vehicle.filename", i));
		
		-- check ONS State
		if ONS then
		    str2 = getXMLString(xmlFile, string.format("vehicle.visible.visible(%d)#objectNIndex", i));
			-- check str2
			node2 = Utils.indexToObject(self.components, str2);
            if node2 == nil then
                --print("[Error::visible.lua] objectNState is true and objectHIndex is a nil value in "..tostring(errorInformations).." (.i3d in .xml)");
                break;
			end;
		end;
		
        -- check all Values
		if input == nil then
		    --print("[Error::visible.lua] a nil inputName in "..tostring(errorInformations).." (.i3d in .xml)");
			break;
		end;
		
		-- indexItems
		local node1 = Utils.indexToObject(self.components, str1);
		
		if node1 == nil then
		    --print("[Error::visible.lua] a nil index in "..tostring(errorInformations).." (.i3d in .xml)");
			break;
		end;
		
		-- workStrings
		self.visibleNodes[i+1] = {};
		self.visibleNodes[i+1].nodeH = node1;
		self.visibleNodes[i+1].nodeONS = ONS;
		-- check ONS State too
		if ONS then
		    self.visibleNodes[i+1].nodeN = node2;
		end;
		self.visibleNodes[i+1].inputName = input;
		self.visibleNodes[i+1].helptext = help;
		self.visibleNodes[i+1].state = false;
		i = i + 1;
	end;
end;

function visible:delete()
end;

function visible:mouseEvent(posX, posY, isDown, isUp, button)
end;

function visible:keyEvent(unicode, sym, modifier, isDown)
end;

function visible:update(dt)
    if self:getIsActiveForInput() then
		for i=1, #self.visibleNodes do
			if InputBinding.hasEvent(InputBinding[self.visibleNodes[i].inputName]) then
				self:swapVisible(i, not self.visibleNodes[i].state);
			end;
		end;
	end;
end;

function visible:swapVisible(id, state, noEventSend)
	changeVisibleEvent.sendEvent(self, id, state, noEventSend);
	self.visibleNodes[id].state = state;
	    setVisibility(self.visibleNodes[id].nodeH, state);
	    -- check ONS State
	    if self.visibleNodes[id].nodeONS then
	        setVisibility(self.visibleNodes[id].nodeN, not state);
	    end;	
end;

function visible:updateTick(dt)
end;

function visible:draw()
	if self:getIsActiveForInput() then
	    for i=1, #self.visibleNodes do
		    if self.visibleNodes[i].helptext then
		        g_currentMission:addHelpButtonText(g_i18n:getText(self.visibleNodes[i].inputName), InputBinding[self.visibleNodes[i].inputName]);
			end;
		end;
	end;
end;

--loadFromAttributes
function visible:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)	
	for i=1, #self.visibleNodes do
		local state = getXMLBool(xmlFile, key..string.format("#visibleNode%d", i));
		if state ~= nil then
			self:swapVisible(i, state, true);
		end;
	end;
	return BaseMission.VEHICLE_LOAD_OK;
end;
-- saveAttributes
function visible:getSaveAttributesAndNodes(nodeIdent)
	local attributes = "";
	for i=1, #self.visibleNodes do
		local key = string.format(" visibleNode%d",i);
		attributes = attributes..key..'="'..tostring(self.visibleNodes[i].state)..'"';
	end;
	return attributes, nil;
end;

-- changeVisibleEvent

changeVisibleEvent = {};
changeVisibleEvent_mt = Class(changeVisibleEvent, Event);

InitEventClass(changeVisibleEvent, "changeVisibleEvent");

function changeVisibleEvent:emptyNew()  
	local self = Event:new(changeVisibleEvent_mt );
	self.className="changeVisibleEvent";
	return self;
end;

function changeVisibleEvent:new(vehicle, id, saveVisibleState)
	local self = changeVisibleEvent:emptyNew()
	self.vehicle = vehicle;
	self.id = id;
	self.saveVisibleState = saveVisibleState;
	return self;
end;

function changeVisibleEvent:readStream(streamId, connection)
	local id = streamReadInt32(streamId);
	self.vehicle = networkGetObject(id);
	self.id = streamReadInt32(streamId);
	self.saveVisibleState = streamReadBool(streamId);
	self:run(connection);
end;

function changeVisibleEvent:writeStream(streamId, connection)
	streamWriteInt32(streamId, networkGetObjectId(self.vehicle));
	streamWriteInt32(streamId, self.id);	
	streamWriteBool(streamId, self.saveVisibleState);
end;

function changeVisibleEvent:run(connection)
	if self.vehicle ~= nil then
		self.vehicle:swapVisible(self.id, self.saveVisibleState, true);
		if not connection:getIsServer() then
			g_server:broadcastEvent(changeVisibleEvent:new(self.vehicle, self.id, self.saveVisibleState), nil, connection, self.vehicle);
		end;
	end;
end;

function changeVisibleEvent.sendEvent(vehicle, id, saveVisibleState, noEventSend)  
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then   
			g_server:broadcastEvent(changeVisibleEvent:new(vehicle, id, saveVisibleState), nil, nil, vehicle);
		else 
			g_client:getServerConnection():sendEvent(changeVisibleEvent:new(vehicle, id, saveVisibleState));
		end;
	end;
end;

